@@ -1,6 +1,8 @@
 # $id$
 #
-'''Simple, elegant HTML generation.
+'''
+Simple, elegant HTML generation
+-------------------------------
 
 To construct HTML start with an instance of ``html.HTML()``. Add
 tags by accessing the tag's attribute on that object. For example::
@@ -15,33 +17,36 @@ If the tag should have text content you may pass it at tag creation time or
 later using the tag's ``.text()`` method (note it is assumed that a fresh
 ``HTML`` instance is created for each of the following examples)::
 
-   >>> p = h.p('hello, world!\n')
-   >>> p.text('more text')
+   >>> p = h.p('hello world!\\n')
+   >>> p.text('more &rarr; text', escape=False)
    >>> h.p
    >>> print h
    <p>hello, world!
-   more text</p>
+   more &rarr; text</p>
    <p>
 
-Any HTML-specific characters (``<>&"``) in the text will be quoted as
-appropriate. Note also that the top-level ``HTML`` object adds newlines
-between tags by default.
+Any HTML-specific characters (``<>&"``) in the text will be escaped for HTML
+safety as appropriate unless ``escape=False`` is passed. Note also that the
+top-level ``HTML`` object adds newlines between tags by default.
 
 If the tag should have sub-tags you have two options. You may either add
 the sub-tags directly on the tag::
 
    >>> l = h.ol
    >>> l.li('item 1')
-   >>> l.li('item 2 > 1')
+   >>> l.li.b('item 2 > 1')
    >>> print h
    <ol>
    <li>item 1</li>
-   <li>item 2 &gt; 1</li>
+   <li><b>item 2 &gt; 1</b></li>
    </ol>
 
 Note that the default behavior with lists (and tables) is to add newlines
-between sub-tags to generate a nicer output. The alternative to the above
-method is to use the containter tag as a context for adding the sub-tags::
+between sub-tags to generate a nicer output.
+
+The alternative to the above method is to use the containter tag as a
+context for adding the sub-tags. The top-level ``HTML`` object keeps track
+of which tag is the current context::
 
    >>> with h.table(border='1'):
    ...   for i in range(2):
@@ -56,6 +61,14 @@ method is to use the containter tag as a context for adding the sub-tags::
 
 Note the addition of an attribute to the ``<table>`` tag.
 
+A variation on the above is to explicitly reference the context variable,
+but then there's really no benefit to using a ``with`` statement. The
+following is functionally identical to the first list construction::
+
+   >>> with h.ol as l:
+   ...   l.li('item 1')
+   ...   l.li.b('item 2 > 1')
+
 You may turn off/on adding newlines by passing ``newlines=False`` or
 ``True`` to the tag (or ``HTML`` instance) at creation time::
 
@@ -70,10 +83,10 @@ You may turn off/on adding newlines by passing ``newlines=False`` or
 This code is copyright 2009 eKit.com Inc (http://www.ekit.com/)
 See the end of the source file for the license of use.
 '''
-__version__ = '1.0'
+__version__ = '1.1'
 
 import unittest
-from cgi import escape
+import cgi
 
 class HTML(object):
     '''Easily generate HTML.
@@ -105,18 +118,23 @@ class HTML(object):
         else:
             self.content.append(e)
         return e
-    def text(self, text):
+    def text(self, text, escape=True):
+        if escape:
+            text = cgi.escape(text)
         # adding text
-        self.content.append(escape(text))
+        if self.top:
+            self.stack[-1].content.append(text)
+        else:
+            self.content.append(text)
     def __call__(self, *content, **kw):
         # customising a tag with content or attributes
         if content:
-            self.content = map(escape, content)
+            self.content = map(cgi.escape, content)
         if 'newlines' in kw:
             # special-case to allow control over newlines
             self.newlines = kw.pop('newlines')
         for k in kw:
-            self.attrs[k] = escape(kw[k], True)
+            self.attrs[k] = cgi.escape(kw[k], True)
         return self
     def __enter__(self):
         # we're now adding tags to me!
@@ -156,6 +174,11 @@ class TestCase(unittest.TestCase):
         h.text('<')
         self.assertEquals(str(h), '&lt;')
 
+    def test_no_escape(self):
+        h = HTML()
+        h.text('<', False)
+        self.assertEquals(str(h), '<')
+
     def test_escape_attr(self):
         h = HTML()
         h.br(a='"')
@@ -172,8 +195,15 @@ class TestCase(unittest.TestCase):
         h = HTML()
         l = h.ol
         l.li('foo')
-        l.li('bar')
-        self.assertEquals(str(h), '<ol>\n<li>foo</li>\n<li>bar</li>\n</ol>')
+        l.li.b('bar')
+        self.assertEquals(str(h), '<ol>\n<li>foo</li>\n<li><b>bar</b></li>\n</ol>')
+
+    def test_subtag_direct_context(self):
+        h = HTML()
+        with h.ol as l:
+            l.li('foo')
+            l.li.b('bar')
+        self.assertEquals(str(h), '<ol>\n<li>foo</li>\n<li><b>bar</b></li>\n</ol>')
 
     def test_subtag_no_newlines(self):
         h = HTML()
